/**********************************************************************
 *	PS2 Keyboard Library
 **********************************************************************
 *
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include "ps2keyb.h"
#include "delay.h"
/*
void delay_loop(uchar __count);
void delay_us(uchar us);
void delay_100us(uchar us100);
void delay_ms(uchar ms);
void delay_100ms(uchar ms100);
 */

/**********************************************************************
 *	
 **********************************************************************
 */


//	PortD
#define  PS2_PORT PORTD      	// 
#define  PS2_PIN  PIND      	// 
#define  PS2_DDR  DDRD       	// 

#define  PS2_DAT   (1<<4)       // PD4: PS2 DATA
#define  PS2_CLK   (1<<5)       // PD5: PS2 CLK

#ifndef F_CPU
#define   F_CPU   8000000    	//CPUNbNg 8MHz
#endif



#define	POWERON_ACK_SEND	(1)	// dONRESETR}h̊ԁA_~[ACK𑗂.
//ȂƃL[{[hڑƌȂ}U[̂...


/**********************************************************************
 *	
 **********************************************************************
 */

void ps2_sendbyte(uchar dat);
int  ps2_recvbyte(void);
void ps2_recvcmd(void);

static	uchar	reset_done=0;

void ps2_sendclk(void)
{
	delay_us(10);		PS2_DDR |=  PS2_CLK;
	delay_us(50);   	PS2_DDR &= ~PS2_CLK;
	delay_us(10);
}

void ps2_init(void)
{
	PS2_PORT &= ~(PS2_DAT|PS2_CLK);
	PS2_DDR  &= ~(PS2_DAT|PS2_CLK);

	delay_100ms(10);			//	Pb.
}

char ps2_recvbit(void)
{
	ps2_sendclk();
	return(PS2_PIN & (PS2_DAT));
}

void ps2_sendbyte(uchar dat)
{
	char parity;
	int i,wdat;

	if((PS2_PIN & (PS2_CLK))==0) {
		return;
	}
	while((PS2_PIN & (PS2_DAT))==0) {
		ps2_recvcmd();
	}

	wdat = (int)dat << 1;

	parity=0;
	for(i=0; i<8; i++){
		if((1<<i) & dat){
			parity++;	
		}
	}
	if((parity & 1)==0){
		//rbg1̐
		wdat |= (1<<9);	//rbg9(peB)=1
	}
	wdat |= (1<<10);	//rbg10(Xgbvrbg)=1

	for(i=0; i<11; i++){	//ʂ11rbgM
		if((1 << i) & wdat){
			PS2_DDR &= ~(PS2_DAT);	//1(HighCs[_X)
		}else{
			PS2_DDR |= (PS2_DAT);	//0
		}
		ps2_sendclk();	//send 1bit clock
		if((PS2_PIN & (PS2_CLK))==0)
			break;	//CLKL̏ꍇA
	}
	PS2_DDR &= ~(PS2_DAT);
	delay_ms(2);
}

// PS2RlN^ 1bytesM
int	ps2_recvbyte(void)
{
	uchar mask=0x01;
	uchar data=0x0;

	if((PS2_PIN & (PS2_CLK))==0)
		return -1;		//CLK=Low̏ꍇA

	if((PS2_PIN & (PS2_DAT))!=0)
		return -1;		//DATHigh̏ꍇA(f[^Ȃ)

	ps2_recvbit();	//startbit

	while(mask != 0) {
		if(ps2_recvbit()) {
			data |= mask;
		}
		mask <<= 1;
	}
	ps2_recvbit();	//parity
	if(ps2_recvbit()){
		PS2_DDR |=  (PS2_DAT);ps2_sendclk();
		PS2_DDR &= ~(PS2_DAT);
	}else{
		return -1;		// Stop Bit Error
	}
	return data;
}

void	ps2_recvcmd(void)
{
	uchar data;
	int c;
	c = ps2_recvbyte();
	if(c<0) return;
	delay_ms(1);	// 1msec
					// *@ɂă^C~OႤ\
	data=c;
	//Mf[^ǂƕԐM
	switch(data) {
	  case 0xff:				//ResetR}h
		ps2_sendbyte(0xFA);		//Ack
		delay_100ms(6);			//600msec
		ps2_sendbyte(0xAA);		//BAT
		reset_done=1;
		break;
	  case 0xf2:				//IDǂݏoR}h
		ps2_sendbyte(0xFA);		//Ack
		ps2_sendbyte(0xab);		//L[{[hID
		ps2_sendbyte(0x83);		//L[{[hID
		break;
	  case 0xed:				//IDǂݏoR}h
		ps2_recvbyte();
		ps2_sendbyte(0xFA);		//Ack
		break;
	  case 0xf3:				//set typematic rate
		ps2_recvbyte();
		ps2_sendbyte(0xFA);		//Ack
		break;
	  case 0xee:				//GR[R}h
		ps2_sendbyte(0xEE);		//
		break;
	  default:
		ps2_sendbyte(0xFA);		//Ack
		break;
	}

#if	POWERON_ACK_SEND
	{
		static	char swnum;
		swnum++;
		if(swnum==0) {
			if(reset_done==0) {
				ps2_sendbyte(0xfa);		/*code(J[\L[)*/
			}
		}
	}
#endif

}

#define FLAG_MAKECODE 0
#define FLAG_BREAKCODE 1

void ps2_sendcode(char swnum,char flag)
{
	if(swnum & 0x80){
		ps2_sendbyte(0xE0);		/*code(J[\L[)*/
		swnum &= 0x7f;
	}

	if(flag == FLAG_BREAKCODE) {
		ps2_sendbyte(0xF0);		/*break code*/
	}
	ps2_sendbyte(swnum);		/*code*/

}

//	f=0	Break
//	f=1	Make
void keypress(char swnum,char f)
{
	if(f==0) {
		ps2_sendcode(swnum,FLAG_BREAKCODE);	// Break Code
	}else{
		ps2_sendcode(swnum,FLAG_MAKECODE);		// Make code
	}
}



